#include "board.h"
#include "objects.h"
#include <string.h>
#include <assert.h>

char pointsDesc[pointsCount][20]= {
"Kill",
"Dead",
"Fired",
"HitTank",
"DestGround",
"HitMissile",
"PointsGround",
"Points"
};

Board::Board( int _width, int _height, int _turns, bool enableGui)
{	
	assert( _width<=MAX_BOARD_SIZE);
	assert( _height<=MAX_BOARD_SIZE);

	turnsProp=_turns;

//otvor fifo do gui
	if (enableGui) {
		fGuiProp = fopen(FGUI_NAME, "w");
		if (fGuiProp==0)
			fprintf(stderr,"Chyba pri otvarani %s\n",FGUI_NAME);
	}

	widthProp=_width;
	heightProp=_height;
	memset(points,0,sizeof(points));

	Object *o;
	memset(map,0,sizeof(map));

	if (fGui()) {  //posleme gui velkost planu
		fprintf(fGui(),"%d %d\n", width(), height() ); fflush(fGui());
	}

//vyrobime okraje mapy
#define DO(x,y) o = new ObjWall(Point((x),(y)), this); o->putToBoard();

	for (int i=0; i<_width; i++)
		for (int j=0; j<BOARD_BORDER; j++) {
			DO(i,j);
			DO(i,_height-j-1);
		}

	for (int j=BOARD_BORDER; j<_height-BOARD_BORDER; j++)
		for (int i=0; i<BOARD_BORDER; i++) {
			DO(i,j);
			DO(_width-i-1,j);
		}

#undef DO

}

Board::~Board()
{
//uvolnime pamat
	for (int i=0; i<width(); i++) 
		for (int j=0; j<height(); j++)
			while (map[i][j])        //na jednej pozicii moze byt viac objektov
				map[i][j]->setDeleted();
	
	freeFreeList();
}

void Board::fillGround()
{
	for (int i=0; i<width(); i++) 
		for (int j=0; j<height(); j++)
			if (map[i][j]==0)	{
				Object *o = new ObjGround( Point(i,j), this);
				o->putToBoard();
			}
}

Object* Board::get(const Point &p) 
{
	return map[p.x][p.y];
}

void Board::clear(const Point &p)
{
	map[p.x][p.y]=0;
}

void Board::put(const Point &p, Object *o)
{
	assert(map[p.x][p.y]==0);
	map[p.x][p.y]=o;
}

void Board::printState(FILE *f, Point bp)
{
	fprintf(f, "%d\n", turns());   //aktualny cas
	fprintf(f, "%d\n", players );  //pocet hracov

	Point lt= bp-Point( STAV_POLOMER, STAV_POLOMER);
	Point size( 2*STAV_POLOMER+1, 2*STAV_POLOMER+1);
			
	for (int j=lt.y; j<lt.y+size.y; j++) {
		for (int i=lt.x; i<lt.x+size.x; i++) {
			assert(map[i][j]);
			fputc( map[i][j]->getCode( Point(i,j) ), f );
			
			map[i][j]->marked=false;           //vsetky objekty si odznacime
		}
		fputc('\n', f);
	}
	fputc('\n', f);

	for (int j=lt.y; j<lt.y+size.y; j++)
		for (int i=lt.x; i<lt.x+size.x; i++) {
			if (!map[i][j]->marked)            //o neoznackovanych vypiseme info
				map[i][j]->printInfo(f, bp); 
			map[i][j]->marked=true;            //tie, ktore sme videli si oznackujeme
		}
			
	fprintf(f, "%d\n", objKoniec);
			
}

void Board::freeFreeList()
{
	for (ObjectList::iterator it= freeList.begin(); it!= freeList.end(); ) {
		delete *it;
		it=freeList.erase(it);
	}
}

void Board::addToFreeList( Object * o)
{
	freeList.push_back( o );
}

void Board::addObject( Object *o )
{	
	active.push_front(o);
	o->putToBoard();
}

int Board::run()
{
	for (; turns(); turnsProp-- ) {
		
		for ( ObjectList::iterator it= active.begin(); it!= active.end(); it++)
			if ( !(*it)->isDeleted() )
				(*it)->step();

		if (fGui()) {
			fprintf(fGui(),"%d %d\n",FG_END_TURN, turns()); fflush(fGui());
		}
		
		//odstran zmazane entity
		for ( ObjectList::iterator it= active.begin(); it!= active.end();)
			if ( (*it)->isDeleted() )
				it=active.erase(it);
			else
				it++;

		freeFreeList();
		
//		printState( stdout, Point(15,15) );
	}
	if (fGui()) {
		fprintf(fGui(),"%d\n",FG_END_GAME); fflush(fGui());
	}
	
	for ( ObjectList::iterator it= active.begin(); it!= active.end(); it++)
		(*it)->setDeleted();

	return 0;
}
