#ifndef OBJECT_H
#define OBJECT_H

#include <list>
#include "types.h"
#include <stdio.h>

class Board;

class Point: public PointStruct {
public:
	Point( int _x=0, int _y=0);
	Point( PointStruct p);

	void operator+=( const Point &p ){ x+=p.x; y+=p.y; };
	void operator-=( const Point &p ){ x-=p.x; y-=p.y; };
	Point operator+( const Point &p ){ return Point(x+p.x, y+p.y); };
	Point operator-( const Point &p ){ return Point(x-p.x, y-p.y); };
	Point operator*( const int k ){ return Point(x*k, y*k); };
};

class Object {
public:
//bp - zakladny bod objektu
//board - hracia plocha
//Konstruktor sa sam vlozi na plochu
	Object(Point _bp, Board *_board, int _type);

	virtual ~Object();

//vykonaj cinnost za jednu casovu jednotku
	virtual void step();

	virtual void putToBoard();
	virtual void removeFromBoard();
	bool onBoard() { return onBoardProp; };

	virtual char getCode( Point p );  //vrati znak pre vypis

	//vypise informacie o objekte na jeden riadok suboru
	//vypisuje svoju poziciu vzhladom na Point
	//zaujimaje iba pre objekty, ktore nechceme, aby sa parsovali z mapy
	virtual void printInfo( FILE *, Point ) {}; 
	
	//ak potomok nema predefinovany putToBoard, tak moze predefinovat toto, a 
	//bude zavolane pri putToBoard
	virtual void putToGui() {}; 
	
	Point bp() {return bpProp;};  //zakladny bod

	virtual void hit(int);        //objekt je zasiahnuty - parameter urcuje, ktory hrac zasiahol
	//vracia true, ked sa objekt zasahom znicil
	
	virtual void setDeleted(); //vybere objeckt z dosky a zaradi do toFree litu dosky
	bool isDeleted(){ return deletedProp; };

	int type() {return typeProp; };
	void setType( int _type ) {typeProp=_type; };

	int id() {return idProp;};  //jednoznacny identifikator objektu

	bool marked;  //Board si tu znaci, ktore objeckty videl v okne, ktore dumpuje
protected:
	Board *board;
	Point bpProp;
	bool onBoardProp; //je objekt umiestneny na doske?

private:
	bool deletedProp;
	int typeProp;
	int idProp;
};

typedef Object *PObject;
typedef std::list<Object*> ObjectList;

#endif //OBJECT_H
