#include "objects.h"
#include "board.h"
#include "types.h"

#include <stdio.h>

ObjWall::ObjWall(Point _bp, Board *_board)
	: Object(_bp, _board, objStena)
{
}

void ObjWall::putToGui()
{
	if (!(board->fGui())) return;
	fprintf(board->fGui(), "%d %d %d %d %s %d\n", id(), type(), bp().x, bp().y, "NONE", 0 );
	fflush(board->fGui());
}

// ****************   Nothing  ******************

ObjNothing::ObjNothing(Point _bp, Board *_board)
	: Object(_bp, _board, objNic)
{
}

void ObjNothing::putToGui()
{
	if (!board->fGui()) return;
	fprintf(board->fGui(), "%d %d %d %d %s %d\n", id(), type(), bp().x, bp().y, "NONE", 0 );
	fflush(board->fGui());
}

// ****************   Ground  ******************

ObjGround::ObjGround(Point _bp, Board *_board)
	: Object(_bp, _board, objZem)
{
}

void ObjGround::hit(int num)
{
	setDeleted(); //na nase miesto pride nic 
	Object *o= new ObjNothing( bp(), board );
	o->putToBoard();

	board->points[num][pointsHitGround]++;     //zapocitame skore
}

// ****************   Home  ******************

ObjHome::ObjHome(Point _bp, Board *_board, int _number)
	: Object( _bp, _board, objDom )
{
	numberProp=_number;
}

void ObjHome::putToBoard()
{
	assert(board);
	for (int i=bp().x-HOME_POLOMER; i<=bp().x+HOME_POLOMER; i++)
		for (int j=bp().y-HOME_POLOMER; j<=bp().y+HOME_POLOMER; j++) {
			assert(board->get( Point(i,j) ) == 0 );
			board->put( Point(i,j), this);
		}
	onBoardProp=1;
	
	if (board->fGui()) { 
		char s[20];
		sprintf(s,"PL%d",number());
		
		fprintf(board->fGui(), "%d %d %d %d %s %d\n", id(), type(), bp().x, bp().y, s, 0);
		fflush(board->fGui());
	}
}

void ObjHome::removeFromBoard()
{
	for (int i=bp().x-HOME_POLOMER; i<=bp().x+HOME_POLOMER; i++)
		for (int j=bp().y-HOME_POLOMER; j<=bp().y+HOME_POLOMER; j++)
			board->clear( Point(i,j));
	onBoardProp=0;
}

char ObjHome::getCode( Point p )
{
	return Object::getCode(p)+number()-1;
}

void ObjHome::printInfo( FILE *f, Point pp )
{
	Point p= bp()-pp;
	fprintf(f, "%d %d %d %d %d\n", objDom, id(), p.x, p.y, number());
}
// ****************   Missile  ******************

ObjMissile::ObjMissile(Point _bp, Board *_board, ObjTank *_tank)
	: Object(_bp, _board, objStrela)
{
	assert(_tank); assert(_board);
	tank=_tank;
	dir=tank->dir();
	numberProp=tank->number();
	under=0;
	home=bp();
}

ObjMissile::~ObjMissile()
{
	assert(under==0); //musime byt odstraneny z mapy
}

void ObjMissile::hit(int)
{
	if (isDeleted()) return;

	//teraz strela iba vybuchne, zmaze sa az pri volani setDeleted
	if (board->fGui()) { 
		fprintf(board->fGui(), "%d %d %d %d %s %d\n", id(), type(), bp().x, bp().y, "EXPLODE", 0);
		fflush(board->fGui());
	}

	setDeleted();     //strela padne po prvej rane

}

void ObjMissile::putToBoard()
{
	assert(board); assert(!under);
	
	Object *o=board->get(bp());
	assert(o);

	switch (o->type()) {
		case objNic :  //cez nic a dom preletime
		case objDom :
			under=o;
			board->clear(bp());
			board->put(bp(), this);
			onBoardProp=1;
			break;

		case objStrela :  //navzajom sa zasiahneme
			o->hit(number());
			hit( ((ObjMissile*) o)->number() );

			board->points[((ObjMissile*) o)->number()][pointsHitMissile]++;  //zapocitame skore obo strelam
			board->points[number()][pointsHitMissile]++;  //zapocitame skore obom strelam
			break;

		case objZem :
			tank->addPointsWeightGround( home ); //body za zaminu dostavame podla toho, kde sme vystrelili

		default:    //vsetko ostatne zasiahneme a zniceme sa
			o->hit(number());
			hit(number());
	}
	

	if (board->fGui() && !isDeleted() ) {
		char s[20];
		sprintf(s,"DIR%d",dir);
		
		fprintf(board->fGui(), "%d %d %d %d %s %d\n", id(), type(), bp().x, bp().y, s, 1);
		fflush(board->fGui());
	}

}

void ObjMissile::removeFromBoard()
{
	assert(board); 

	if (under) {	
		board->clear(bp());
		board->put(bp(), under);
		under=0;
		onBoardProp=0;
	}
	
}

void ObjMissile::step()
{
	assert(under);

	removeFromBoard();
	bpProp+= smer[ dir ];
	putToBoard();

	assert(under || isDeleted() );
}

char ObjMissile::getCode( Point )
{
	return strelaKody[dir];
}

void ObjMissile::printInfo( FILE *f, Point pp)
{
	Point p= bp()-pp;
	fprintf(f, "%d %d %d %d %d %d\n", objStrela, id(), p.x, p.y, dir, number());
}

void ObjMissile::setDeleted()
{
	Object::setDeleted();

  tank->firePoped();
}
// 
// ***************   Tank    ********************
 
ObjTank::ObjTank(Point _home, Board *_board, int _number)
	: Object(_home, _board, objTank)
{
	assert(_board);

	numberProp=_number;
	home=_home;
	explode=0;

	memset(under, 0, sizeof(under));
	memset(statTypes, 0, sizeof(statTypes));
	memset(statHome, 0, sizeof(statHome));

	dirProp=smerS;
	en=TANK_EN;
	hp=TANK_HP;
	fire_timeout=0;
	fires=MAX_STRIEL_OD_HRACA;
	sleep=TANK_CAS_RESET;
	mustPut=0;

//otvor fifa
	char fn[50];
	sprintf(fn, FTOPLAYER_NAME, number());
	ftoplayer = fopen(fn, "w");
	if (!ftoplayer)
		fprintf(stderr,"Chyba pri otvarani %s\n",fn);

	sprintf(fn, FFROMPLAYER_NAME, number());
	ffromplayer = fopen(fn, "r");
	if (!ffromplayer)
		fprintf(stderr,"Chyba pri otvarani %s\n",fn);
}

ObjTank::~ObjTank()
{
	if ( ! (!ffromplayer || !ftoplayer || feof(ffromplayer) || feof(ftoplayer) )) {

		fprintf(ftoplayer, "%d\n", FP_KONIEC);
		fflush(ftoplayer);
	}

	if (ftoplayer) 
		fclose(ftoplayer);
	if (ffromplayer) 
		fclose(ffromplayer);
}

void ObjTank::reset()   //umiestni tank do domu, doplni en a hp
{
	removeFromBoard();

	dirProp=smerS;
	en=TANK_EN;
	hp=TANK_HP;
	sleep=TANK_CAS_RESET;
	bpProp=home;

	mustPut=1;
	
	board->points[number()][pointsDead]++;  //zapocitame si smrt
	board->points[number()][pointsPointsAll]+=POINTS_DEAD;  //zapocitame si smrt
}

void ObjTank::hit(int num)
{
	board->points[num][pointsHitTank]++;  //zapocitame superovi zasah

	hp-=TANK_ENHP_ZASAH;
	if (hp<0) {
		if (en>0) {
			en+=hp;
			if (en<=0) {
				board->points[num][pointsKill]++;  //zapocitame superovi nasu smrt
				board->points[num][pointsPointsAll]+=POINTS_KILL;  //zapocitame superovi nasu smrt
				
				explode=4;
				if (board->fGui()) { 
					fprintf(board->fGui(), "%d %d %d %d %s %d %d %d %d %d\n", id(), type(), bp().x, bp().y, "EXPLODE", 1, 
						number(), en, hp, board->points[number()][pointsPointsAll]);

					fflush(board->fGui());
				}
			}
			hp=0;
		}
	}

}

void ObjTank::step()
{
	if (explode>1) { explode--; return; }
	if (explode==1) { explode=0; reset(); return;}
		
	if (en<0) { 
		explode=4; 
		if (board->fGui()) { 
			fprintf(board->fGui(), "%d %d %d %d %s %d %d %d %d %d\n", id(), type(), bp().x, bp().y, "EXPLODE", 1, 
				number(), en, hp, board->points[number()][pointsPointsAll]);
			fflush(board->fGui());
		}
		return;
	}

	if (mustPut) {
		if (canMove(bp(), dir())) {  //ak tam niekto netrci
			mustPut=0;
			putToBoard();
		} else
			return;   //inac musime pockat			
	}
	assert(onBoard());

	sleep--;
	sendState();

	if (sleep<=0) { //nespime

		removeFromBoard();

		Point nbp=bp();
		int ndir=dir();
		
		int move= moveRequest();
		switch ( move ) {
			case TP_TAH_DOPREDU:
				nbp+=smer[dir()]; break;
			case TP_TAH_DOZADU:
				nbp-=smer[dir()]; break;
			case TP_TAH_DOLAVA:
				ndir= (ndir-1 + smerPocet) % smerPocet;	break;
			case TP_TAH_DOPRAVA:
				ndir= (ndir+1 + smerPocet) % smerPocet;	break;
			case TP_TAH_NIC:
				break;
			default:
				assert(false);
		}

		if (canMove(nbp, ndir)) {
			bpProp=nbp; dirProp=ndir;
		} else
			move = TP_TAH_NIC;   //ak je zly pohyb, nerobime nic
		
		sleep+=tankCasy[move];
		en-=tankEn[move];

		putToBoard();
	}

	if (fires>0 && fire_timeout<=0)
		if (fireRequest())
			fire();

	if (fire_timeout>0)
		fire_timeout--;
	
	//doplnanie energie a stitu
	en+= statTypes[objDom]*TANK_EN_DOPLNANIE;  if (en>TANK_EN) en=TANK_EN;
	hp+= statHome[number()]*TANK_HP_DOPLNANIE;   if (hp>TANK_HP) hp=TANK_HP;

	assert(sleep>0);
}

void ObjTank::removeFromBoard()
{
	if (!onBoard()) return;

	for (int i=0; i<TANK_ROZMER; i++)
		for (int j=0; j<TANK_ROZMER; j++) { //tam, kde sme zapisali seba, vratime povodny obsah
			if (under[i][j]) {
				Point p( bp()+Point(i,j)+ tankOffset );

				board->clear(p);
				board->put(p, under[i][j]);

				under[i][j]=0;
			}
		}
	
	memset(statTypes, 0, sizeof(statTypes));
	memset(statHome, 0, sizeof(statHome));
	onBoardProp=0;

}

void ObjTank::putToBoard()
{
	assert(onBoard()==0);

	for (int i=0; i<TANK_ROZMER; i++)
		for (int j=0; j<TANK_ROZMER; j++) { //tam, kde sme, zapiseme seba do board
			assert(under[i][j]==0);
			if (tankTvary[dir()][j][i]) {  //definicia tvaru ma otocene suradnice
				Point p( bp()+Point(i,j)+ tankOffset );

				Object *o=board->get(p); 

				while (o->type()!=objNic && o->type()!=objDom) {  //pod nami moze byt len toto
					switch (o->type()) {
						case objStrela:      //strela pod nami, nas zasiahne
							hit( ((ObjMissile*) o)->number() ); 
							o->setDeleted();
							o=board->get(p); 
							break;

						case objZem:         //zeminu zasiahneme my
							o->hit(number());
							o=board->get(p); 
							sleep+=TANK_CAS_BAGER;
							en-=TANK_EN_BAGER;

							//body od vzdialenosti
							addPointsWeightGround(bp());
							break;

						default:             //nic ostatne pod nami nesmie byt
							assert(false);
					}
				}

				under[i][j]=o;
				
				statTypes[o->type()]++;
				if (o->type()==objDom) 
					statHome[ ((ObjHome*) o)->number()]++;

				board->clear(p);
				board->put(p, this);

				assert(under[i][j]);
			}
		}
				
	onBoardProp=1;
	
	if (board->fGui()) { 
		char s[20];
		sprintf(s,"%dDIR%d", number(), dir());
		
		fprintf(board->fGui(), "%d %d %d %d %s %d %d %d %d %d\n", id(), type(), bp().x, bp().y, s, 1, 
				number(), en, hp, board->points[number()][pointsPointsAll]);
		fflush(board->fGui());
	}
}

void ObjTank::addPointsWeightGround(Point p)
{
	int dx=abs( p.x - home.x);
	int dy=abs( p.y - home.y);
	int d = dx>dy?dx:dy;

	board->points[number()][pointsWeightGround]+= 1+d/POINTS_GROUND_DIVIDER;  //zapocitame skore
	board->points[number()][pointsPointsAll]+= 1+d/POINTS_GROUND_DIVIDER;  //zapocitame skore
}
char ObjTank::getCode( Point p )
{
	return Object::getCode(p)+number()-1;
}

void ObjTank::printInfo( FILE *f, Point pp)
{
	Point p= bp()-pp;
	fprintf(f, "%d %d %d %d %d %d %d %d %d %d\n", objTank, id(), p.x, p.y, dir(), number(), en, hp, sleep, fires);
}

bool ObjTank::canMove( Point bp, int dir )
{
	for (int i=0; i<TANK_ROZMER; i++)
		for (int j=0; j<TANK_ROZMER; j++)
			if (tankTvary[dir][j][i]) {  //definicia tvaru ma otocene suradnice
				Point p( bp+Point(i,j)+ tankOffset );

				Object *o=board->get(p);
				if (o->type()==objStena || o->type()==objTank)
					return false;
			}

	//neda sa vojst taziskom do stredu cudzieho domceku
	Object *o=board->get(bp);
	if (o->type()==objDom && ( ( (ObjHome *) o)->number()!=number()) )
		return false;
		
	return true;
}

void ObjTank::fire()
{
	Object *o = new ObjMissile( bp() + Point(smer[dir()])*((TANK_ROZMER/2)+1), 
			board, this);
	board->addObject(o);
	
	board->points[number()][pointsFired]++;  //zapocitame vystrelenu strelu
	fires--;
	fire_timeout=TANK_CAS_VYSTREL;
	en-=TANK_EN_VYSTREL;
}

void ObjTank::sendState()
{
	if (!ffromplayer || !ftoplayer || feof(ffromplayer) || feof(ftoplayer) ) 
		return;

	fprintf(ftoplayer, "%d\n", FP_STAV);
	fprintf(ftoplayer, "%d %d\n", bp().x - home.x, bp().y - home.y);
	board->printState(ftoplayer, bp());
	fflush(ftoplayer);

	char buf[255];
	fgets(buf, 255, ffromplayer);
}

int ObjTank::moveRequest()
{
	int out;
	char buf[100];

	if (!ffromplayer || !ftoplayer || feof(ffromplayer) || feof(ftoplayer) ) 
		return TP_TAH_NIC;

	fprintf(ftoplayer, "%d\n", FP_TAHAJ);
	fflush(ftoplayer);

	fgets(buf, 100, ffromplayer);
	if ( sscanf(buf, " %d", &out)!=1 ||
		out<0 || out>=TP_TAH_POCET) {
		fprintf(stderr,"Player %d vratil nekorektnu odpoved na tah\n", number());
		out=TP_TAH_NIC;
	}
	return out;
}

bool ObjTank::fireRequest()
{
	int out;
	char buf[100];

	if (!ffromplayer || !ftoplayer || feof(ffromplayer) || feof(ftoplayer) ) 
		return TP_VYSTREL_NIC;

	fprintf(ftoplayer, "%d\n", FP_VYSTRELIT_);
	fflush(ftoplayer);


	fgets(buf, 100, ffromplayer);
	if ( sscanf(buf, " %d", &out)!=1 ||
		out<0 || out>=TP_VYSTREL_POCET) {
		fprintf(stderr,"Player %d vratil nekorektnu odpoved na strielanie\n", number());
		out=TP_VYSTREL_NIC;
	}
	return out==TP_VYSTREL_STRELIT;
}

//pri zruseni posleme cez fifo spravu, ze koncime
void ObjTank::setDeleted()
{
	Object::setDeleted();
}
