#ifndef OBJECTS_H
#define OBJECTS_H

#include "object.h"
#include "config.h"

class ObjWall : public Object {
public:
	ObjWall(Point _bp, Board *_board);
	virtual void putToGui();
};

class ObjNothing : public Object {
public:
	ObjNothing(Point _bp, Board *_board);
	virtual void putToGui();
};

class ObjGround : public Object {
public:
	ObjGround(Point _bp, Board *_board);

	virtual void hit(int);
};

class ObjHome : public Object {
public:
	ObjHome(Point _bp, Board *_board, int _number);

	virtual void putToBoard();
	virtual void removeFromBoard();

	virtual char getCode( Point p );
	virtual void printInfo( FILE *, Point );

	int number() { return numberProp; };
private:
	int numberProp;
};

class ObjTank;

class ObjMissile : public Object {
public:
	ObjMissile(Point _bp, Board *_board, ObjTank *_tank);
	virtual ~ObjMissile();

	virtual void hit(int);
	virtual void step();

	virtual void putToBoard();
	virtual void removeFromBoard();
	
	virtual char getCode( Point p );
	virtual void printInfo( FILE *, Point );
 	
	virtual void setDeleted(); //pri zruseni upovedomime vlastnika
	
	int number() { return numberProp; };
protected:
	int dir, numberProp;
	Object *under; //co je pod nami?
	ObjTank *tank; //tank, ktory ma vystrelil
	Point home;  //to sme boli pocati
};

class ObjTank : public Object {
public:
	ObjTank(Point _home, Board *_board, int _number);
	~ObjTank();

	void reset();    //umiestni tank do domu, doplni en a hp

	virtual void hit(int num);
	virtual void step();

	virtual void putToBoard();
	virtual void removeFromBoard();

	virtual char getCode( Point p );
	virtual void printInfo( FILE *, Point );

	void firePoped(){ fires++; }   //volane strelou, ked vybuchne
	
	virtual void setDeleted(); //pri zruseni posleme cez fifo spravu, ze koncime
	void addPointsWeightGround(Point p); //pridaj body za znicenu zeminu
	
	int number() { return numberProp; };
	int dir() { return dirProp; };
protected:
	int numberProp;
	int dirProp;
	int en, hp;  //energia, hitpointy
	Point home;  //dom - miesto, kde sa objavime, ked nas zabiju
	int sleep;   //kolko casovych jednotiek ma spat
	int fire_timeout; //kolko sekund nemozme vystrelit
	int fires; //kolko striel moze este vystrelit (je obmedzeny pocet striel na obrazovke od jedneho hraca)
	int mustPut; //ak nahodou nie sme posadeny na ploche, tato premenna ma hodnotu 1
	int explode; //kolko tahov mam nic nerobit

	PObject under[TANK_ROZMER][TANK_ROZMER];  //co je pod nami?
	
	int statTypes[objPocet];  //statistika objektov pod nami
	int statHome[MAX_PLAYERS];

	bool canMove( Point bp, int dir );  //moze sa tank presunut na toto policko?
	
	void sendState();  //posle hracovi info o tom, co vidi
	int moveRequest(); //vrati hracov tah
	bool fireRequest(); //chce hrac vystrelit?

	void fire();

	FILE *ftoplayer, *ffromplayer;
};

#endif //OBJECTS_H
