/*--License:
	Kyra Sprite Engine
	Copyright Lee Thomason (Grinning Lizard Software) 2001-2002
	www.grinninglizard.com/kyra
	www.sourceforge.net/projects/kyra

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	The full text of the license can be found in license.txt
*/


#include "SDL.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "kyra.h"
//#include <textbox.h>
#include "data.h"
#include <sys/types.h>
#include <sys/wait.h>

#include "config.h"

#define SDL_TIMER_EVENT ( SDL_USEREVENT + 0 )
const int TIMER_INTERVAL = 80;

const int SCREENX = 1024;
const int SCREENY = 768;
int SOUNDS_PLAYED = 0;

#define FIELD_SIZE 8        //pocet pixelov na strane jedneho stvroceka enginu

double mapScale = 0.5;   //pomer medzi detailom a mapkou

#define MAIN_VIEW_SIZE 610
#define TANK_VIEW_SIZE (19*FIELD_SIZE)
#define TANK_VIEW_VER_SPACE 10

#define MAIN_VIEW_LEFT ((SCREENX-MAIN_VIEW_SIZE)/2)
#define MAIN_VIEW_RIGHT (MAIN_VIEW_LEFT+MAIN_VIEW_SIZE)

#define TANK_VIEW_HOR_SPACE ( (MAIN_VIEW_LEFT-TANK_VIEW_SIZE)/2)
#define TANK_VIEW_LEFT(i)  \
	( ((i-1)%2 ==0 ) ? \
	  TANK_VIEW_HOR_SPACE : \
	  TANK_VIEW_HOR_SPACE+MAIN_VIEW_RIGHT \
	 )
#define TANK_VIEW_RIGHT(i) (TANK_VIEW_LEFT(i)+TANK_VIEW_SIZE)

#define TANK_VIEW_UP(i) (TANK_VIEW_VER_SPACE + (TANK_VIEW_VER_SPACE + TANK_VIEW_SIZE)*((i-1)/2))
#define TANK_VIEW_DOWN(i) (TANK_VIEW_UP(i)+TANK_VIEW_SIZE)
	
#define POS_NOTHING_X -10000
#define POS_NOTHING_Y 10000
	
#define POS_BODY_X   -10000
#define POS_BODY_Y    -10000

#define BODY_LEFT   0    
#define BODY_TOP    615
#define BODY_RIGHT  (SCREENX)
#define BODY_DOWN   (SCREENY)
	
	
#define BODY_TEXT_LEFT   550    

FILE *fgui;

void closeFifos()
{
  if (fgui) fclose(fgui);
}

bool openFifos()
{
  fgui = fopen(FGUI_NAME, "r");
  if (!fgui) {
    fprintf(stderr,"Chyba pri otvarani fifa\n");
    closeFifos();
  }
	return (fgui);
}


Uint32 TimerCallback(Uint32)
{
	SDL_Event event;
	event.type = SDL_TIMER_EVENT;

	SDL_PeepEvents( &event, 1, SDL_ADDEVENT, 0 );
	return TIMER_INTERVAL;
}

KrEngine* engine;
KrSpriteResource *tankRes, *fireRes, *wallRes, *nothingRes, *homeRes, *bgRes;
KrFontResource *textRes;

#define BODY_COUNT 4
KrTextBox *body[BODY_COUNT];

void play(char *)
{
/*	char s2[100];
	
	SOUNDS_PLAYED++;
	sprintf(s2,"cp gui/snd/%s spool/%08d.wav",file,SOUNDS_PLAYED);
	system(s2);*/
}

void hadleEngineEvent(char *s)
{
	int id, type, x, y, time;
	char action[20];
	KrSprite* sprite;
	
	sscanf(s,"%d %d %d %d %s %d",&id, &type, &x, &y, action, &time);

	sprite= (KrSprite*) engine->Tree()->FindNodeById(id);
	
	if (sprite==0) {
		switch (type) {
		case objTank:
			sprite = new KrSprite( tankRes );
			sprite->SetZDepth(10);
			break;
		case objStrela:
			sprite = new KrSprite( fireRes );
			sprite->SetZDepth(12);
			break;
		case objStena:
			sprite = new KrSprite( wallRes );
			sprite->SetZDepth(2);
			break;
		case objNic:
			sprite = new KrSprite( nothingRes );
			sprite->SetZDepth(3);
			break;
		case objDom:
			sprite = new KrSprite( homeRes );
			sprite->SetZDepth(1);
			break;
		default:
			sprite=0;
		}
		if (sprite) {
			sprite->SetNodeId( id );
			engine->Tree()->AddNode( 0, sprite );
		}
	}
	if (sprite) {

		if (x<0 && y<0) {
			engine->Tree()->DeleteNode( sprite );
		} else {
			sprite->SetAction(action);
			sprite->SetPos( x*FIELD_SIZE, y*FIELD_SIZE );
		}

		if (type==objTank) {  //posunieme jeho view, upravime statistiky
			int number, en, hp, points;
			sscanf(s,"%d %d %d %d %s %d %d %d %d %d",&id, &type, &x, &y, action, &time, &number, &en, &hp, &points );
			engine->Tree()->Root()->SetPos(-(x-STAV_STRANA/2)*FIELD_SIZE+(FIELD_SIZE+1)/2, 
					-(y-STAV_STRANA/2)*FIELD_SIZE+(FIELD_SIZE+1)/2, number);

			char s[30]; 
			sprintf(s,"PLAYER%d",number); body[0]->SetTextChar(s, number);
			sprintf(s,"%d",en); body[1]->SetTextChar(s, number);
			sprintf(s,"%d",hp); body[2]->SetTextChar(s, number);
			sprintf(s,"%d",points); body[3]->SetTextChar(s, number);
		}

		if (type==objTank && strcmp(action,"EXPLODE")==0)
			play("tank_exploded.wav");
	}
}

  
void initEngine(SDL_Surface *screen)
{
	KrRect r[2+MAX_PLAYERS];
	int players=MAX_PLAYERS;

	r[0].Set( MAIN_VIEW_LEFT, 0, MAIN_VIEW_RIGHT, MAIN_VIEW_SIZE);
//	printf("%d %d %d %d\n", MAIN_VIEW_LEFT, 0, MAIN_VIEW_RIGHT, MAIN_VIEW_SIZE);
	for (int i=1; i<=players; i++) {
		r[i].Set(TANK_VIEW_LEFT(i), TANK_VIEW_UP(i), TANK_VIEW_RIGHT(i), TANK_VIEW_DOWN(i) );
//		printf("%d %d %d %d\n", TANK_VIEW_LEFT(i), TANK_VIEW_UP(i), TANK_VIEW_RIGHT(i), TANK_VIEW_DOWN(i));
	}
	r[players+1].Set( BODY_LEFT, BODY_TOP, BODY_RIGHT, BODY_DOWN);
	
	engine = new KrEngine( screen, players+2, r , 0 );

	engine->Tree()->Root()->SetScale(mapScale, mapScale, 0);  //mapka je zmensena

	for (int i=1; i<=players; i++)
		engine->Tree()->Root()->SetPos(POS_NOTHING_X, POS_NOTHING_Y, i);    //hracov umiestnime niekde dobludu prec, nech je tam cierno
	
	engine->Tree()->Root()->SetPos(POS_BODY_X, POS_BODY_Y, players+1);    //umiestnime body v engine

	engine->Draw();
	
	// Load the dat file.
	if ( !engine->Vault()->LoadDatFile( "gui/data.dat" ) )
	{
		printf( "Error loading the data.dat file\n" );
		exit( 255 );
	}

	// Get the resources
	tankRes = engine->Vault()->GetSpriteResource( KR_TANK );
	GLASSERT( tankRes );
	
	fireRes = engine->Vault()->GetSpriteResource( KR_FIRE );
	GLASSERT( fireRes );
	
	homeRes = engine->Vault()->GetSpriteResource( KR_HOME );
	GLASSERT( homeRes );
	
	nothingRes = engine->Vault()->GetSpriteResource( KR_NOTHING );
	GLASSERT( nothingRes );

	wallRes = engine->Vault()->GetSpriteResource( KR_WALL );
	GLASSERT( wallRes );

	bgRes = engine->Vault()->GetSpriteResource( KR_BG );
	GLASSERT( bgRes );

	engine->Vault()->CacheScale(mapScale, mapScale);

	
	textRes = engine->Vault()->GetFontResource( KR_TEXTY );
	GLASSERT( textRes );

}

//int main( int argc, char *argv[] )
int main( int, char ** )
{
	SDL_Surface* screen;

	const SDL_version* sdlVersion = SDL_Linked_Version();
	if ( sdlVersion->minor < 2 )
	{
		printf( "SDL version must be at least 1.2.0" );
		exit( 254 );
	}

	/* Initialize the SDL library */
	if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER|SDL_INIT_NOPARACHUTE) < 0 ) {
		printf( "Couldn't initialize SDL: %s\n",SDL_GetError());
		exit(255);
	}

	/* Create a display for the image */
	screen = SDL_SetVideoMode( SCREENX, SCREENY, 0, SDL_HWSURFACE );
	if ( screen == NULL ) {
		exit(3);
	}



	SDL_Event event;
	bool done = false;
	// Start timing!
	SDL_SetTimer( TIMER_INTERVAL, TimerCallback );

	if (!openFifos()) done=true;
	
	{
	int sizex, sizey;
	fscanf(fgui," %d %d ",&sizex, &sizey);  //vycentrujeme pohlad na hlavne okno
	
	mapScale= MAIN_VIEW_SIZE/( (sizex>sizey?sizex:sizey) *FIELD_SIZE*1.0);
		
	initEngine(screen);
	engine->Tree()->Root()->SetPos( 
			(int) (MAIN_VIEW_SIZE - sizex*FIELD_SIZE*mapScale)/2, 
			(int) (MAIN_VIEW_SIZE - sizey*FIELD_SIZE*mapScale)/2, 0);
	
	//dame zeminu ako pozadie
	KrSprite* sprite = new KrSprite( bgRes );
	sprite->SetPos( sizex*FIELD_SIZE/2, sizey*FIELD_SIZE/2 );
	engine->Tree()->AddNode( 0, sprite );
	}
	
	int body_left[BODY_COUNT+1]={0,150,250,350,450};
	for (int i=0; i<BODY_COUNT; i++) {
		if (i==0)
			body[i]= new KrTextBox( textRes, body_left[i+1]-body_left[i],300, 0, (KrTextBox::Alignment) 0);
		else
			body[i]= new KrTextBox( textRes, body_left[i+1]-body_left[i],300, 0, (KrTextBox::Alignment) 2);
		
		engine->Tree()->AddNode(0, body[i]);

		body[i]->SetPos(-POS_BODY_X+BODY_TEXT_LEFT+body_left[i], -POS_BODY_Y);
	}

	body[1]->SetTextChar("EN",0);
	body[2]->SetTextChar("HP",0);
	body[3]->SetTextChar("BODY",0);

//	body[2]->SetTextChar("ahoj",4);
//	body[3]->SetTextChar("45689",5);
	

	printf("Starting\n");

	while (!done) {
		bool timerDone=0;	
		char s[1000];
		int id;
		
		while (1) {
			if (feof(fgui)) done=1;
			fgets(s,1000,fgui);
//			fputs(s,stderr);
			sscanf(s," %d",&id);

			if (id==FG_END_TURN) {
				char ss[30];
				sscanf(s,"%d %s",&id, ss);
				body[0]->SetTextChar(ss,0);

				
				break;
			}
			else if (id==FG_END_GAME) {
				done=1;
				break;
			} else if (id>=0)
				hadleEngineEvent(s );
			
		}

		engine->Draw();

		while( !done && SDL_PollEvent(&event) )
		{
			if ( event.type == SDL_QUIT )
				break;

			switch(event.type)
			{


			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_q:
					done = true;
					break;
				case SDLK_f:
					SDL_WM_ToggleFullScreen(screen);
					break;
				default:
					;
				}
				break;
										
			case SDL_TIMER_EVENT:
				if (!timerDone) {
					timerDone=1;
				}
				break;

			default:
				break;
			}
		}

	}
	
//	while (!feof(fgui))   //odignorujeme povely o zmazani objektov
//		fgetc(fgui);
	
	closeFifos();
	delete engine;

	SDL_Quit();	
	return 0;
}

