#include <stdlib.h>
#include "heap.h"


void heap_up(struct Heap *heap, u_int32_t x)
{
  void *data;
  
  data=heap->base[x];
  
  while(x>1 && heap->cmp(data, heap->base[x>>1])<0)
  {
    heap->base[x]=heap->base[x>>1];
    *((int *)heap->base[x])=x;
    x>>=1;
  }

  heap->base[x]=data;
  *((int *)heap->base[x])=x;
}


void heap_down(struct Heap *heap, u_int32_t x)
{
  void *data;

  data=heap->base[x];

  while((x<<1)<=heap->count)
  {
    x<<=1;

    if(x+1<=heap->count && heap->cmp(heap->base[x], heap->base[x+1])>0)
      x++;
    
    if(heap->cmp(heap->base[x], data)>0)
    {
      x>>=1;
      break;
    }

    heap->base[x>>1]=heap->base[x];
    *((int *)heap->base[x>>1])=x>>1;
  }
  
  heap->base[x]=data;
  *((int *)heap->base[x])=x;
}


struct Heap *heap_new(int32_t (*cmp) (void *, void *), void (*del) (void *))
{
  struct Heap *heap;
  
  heap=malloc(sizeof(struct Heap));
  heap->count=0;
  heap->alloc=HEAP_MALLOC;
  heap->cmp=cmp;
  heap->del=del;
  heap->base=malloc((heap->alloc+1)*sizeof(void *));
  
  return heap;
}


void heap_free(struct Heap *heap)
{
  while(heap->count)
    heap_delete(heap);
  
  free(heap->base);
  free(heap);
}


void heap_insert(struct Heap *heap, void *data)
{
  heap->count++;
  if(heap->alloc<heap->count)
  {
    heap->alloc+=HEAP_MALLOC;
    heap->base=realloc(heap->base, (heap->alloc+1)*sizeof(void *));
  }
  heap->base[heap->count]=data;
  *((int *)heap->base[heap->count])=heap->count;
  heap_up(heap, heap->count);
}


void *heap_extract(struct Heap *heap)
{
  if(heap->count)
    return heap->base[1];
  
  return NULL;
}


void heap_delete(struct Heap *heap)
{
  if(!heap->count)
    return;
  
  if(heap->del)
    heap->del(heap->base[1]);
  heap->base[1]=heap->base[heap->count];
  if(heap->count>1)
    *((int *)heap->base[1])=1;
  heap->count--;
  if(heap->alloc>=heap->count+2*HEAP_MALLOC)
  {
    heap->alloc-=HEAP_MALLOC;
    heap->base=realloc(heap->base, (heap->alloc+1)*sizeof(void *));
  }
  if(heap->count)
    heap_down(heap, 1);
}
