#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include "../com/config.h"
#include "types.h"
#include "player.h"

static FILE *fin, *fout;
int player_number=-1;


static char *va_list2str(va_list va, char *message)
{
  char *buffer;
  int len;

  len=vsnprintf(NULL, 0, message, va)+1;
  buffer=malloc(len);
  vsnprintf(buffer, len, message, va);

  return buffer;
}
 
 
static char *va_arg2str(char *message, ...)
{
  va_list va;

  va_start(va, message);
  message=va_list2str(va, message);
  va_end(va);

  return message;
}


void openFifos()
{
  char *c, *d;

  c=va_arg2str(FTOPLAYER_NAME, player_number);
  d=va_arg2str(FFROMPLAYER_NAME, player_number);
  
  fin=fopen(c, "r");
  fout=fopen(d, "w");
  
  free(d);
  free(c);
}


void closeFifos()
{
  fclose(fin);
  fclose(fout);
}


void readState(struct TStav *stav)
{
  int i, j;
  TypyObjektov x;
  struct TTank *tank;
  struct TStrela *strela;
  struct TDom *dom;
  
  fscanf(fin, "%d %d ", &stav->domov.x, &stav->domov.y);
  fscanf(fin, "%d ", &stav->cas);
  fscanf(fin, "%d", &stav->hracov);
  
  fgetc(fin);
  for(i=0; i<STAV_STRANA; i++)
  {
    for(j=0; j<STAV_STRANA; j++)
      stav->vidim[i][j]=fgetc(fin);
    fgetc(fin);
  }
  
  stav->objekty_pocet=0;
  while(1)
  {
    fscanf(fin, "%d", (int *)&x);
    if(x==objKoniec)
      break;
    
    stav->objekty[stav->objekty_pocet].typ=x;
    switch(x)
    {
      case objTank:
        tank=&stav->objekty[stav->objekty_pocet].obj.objTank;
        fscanf(fin, "%d %d %d %d %d %d %d %d %d ", &tank->id, &tank->poz.x, &tank->poz.y, &tank->dir, &tank->number, &tank->en, &tank->hp, &tank->spi, &tank->striel);
        if(tank->number==player_number)
          memcpy(&stav->ja, tank, sizeof(struct TTank));
        break;
      case objStrela:
        strela=&stav->objekty[stav->objekty_pocet].obj.objStrela;
        fscanf(fin, "%d %d %d %d %d ", &strela->id, &strela->poz.x, &strela->poz.y, &strela->dir, &strela->number);
        break;
      case objDom:
        dom=&stav->objekty[stav->objekty_pocet].obj.objDom;
        fscanf(fin, "%d %d %d %d ", &dom->id, &dom->poz.x, &dom->poz.y, &dom->number);
        break;
      default:
        fprintf(stderr, "Zly typ objektu -- %d.\n", x);
        exit(1);
        break;
    }
    
    stav->objekty_pocet++;
  }
  
  fprintf(fout, "%s\n", vratHlasku());
  fflush(fout);
}


void sendMove()
{
  int x=vratTah();
  if (x<0) x=0;
  if (x>=TP_TAH_POCET) x=0;

  fprintf(fout, "%d\n", x);
  fflush(fout);
}


void sendFire()
{
  fprintf(fout, "%d\n", mamStrelit()?TP_VYSTREL_STRELIT:TP_VYSTREL_NIC);
  fflush(fout);
}


int main(int argc, char **argv)
{
  int x;
  struct TStav stav;
  
  if(argc!=2)
  {
    printf("Usage: \n");
    printf(" %s number\n", argv[0]);
    printf("\n");
    printf(" fifos should be in the current directory\n");
    printf("\n");
    return 1;
  }

  player_number=atol(argv[1]);
  
  openFifos();
  init(74);
  
  while(!feof(fin))
  {
    fscanf(fin, "%d", &x);
    
    if(x==FP_KONIEC)
      break;
    switch(x)
    {
      case FP_STAV:
        readState(&stav);
        novyStav(&stav);
        break;
      case FP_TAHAJ:
        sendMove();
        break;
      case FP_VYSTRELIT_:
        sendFire();
        break;
    }
  }
  
  done();
  closeFifos();

  return 0;
}
